import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:pmd_lab/domain/models/card.dart';
import 'package:pmd_lab/presentation/details_page/details_page.dart';
import 'package:pmd_lab/repositories/api_interface.dart';
import 'package:pmd_lab/repositories/mock_repository.dart';
import 'package:pmd_lab/repositories/potter_repository.dart';
part 'card.dart';

void main() {
  runApp(const MyApp());
}

class MyApp extends StatelessWidget {
  const MyApp({super.key});

  @override
  Widget build(BuildContext context) {
    return MaterialApp(
      title: 'Flutter Demo',
      theme: ThemeData(
        // colorScheme: ColorScheme.fromSeed(seedColor: Colors.black),                                                                                                                  colorScheme: ColorScheme.fromSeed(seedColor: Colors.deepPurple),
        useMaterial3: true,
      ),
      home: const MyHomePage(title: 'Flutter rocks!'),
    );
  }
}

class MyHomePage extends StatefulWidget {
  const MyHomePage({super.key, required this.title});

  final String title;

  @override
  State<MyHomePage> createState() => _MyHomePageState();
}

class _MyHomePageState extends State<MyHomePage> {
  final Color _color = Colors.orangeAccent;

  @override
  void initState(){
    WidgetsBinding.instance.addPostFrameCallback( (_) {
      ScaffoldMessenger.of(context).showSnackBar(SnackBar(
        content: Text('Hello', style: Theme
            .of(context)
            .textTheme
            .bodyLarge,),
        backgroundColor: Colors.orangeAccent,
        duration: const Duration(seconds: 1),
      ));
    });
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: AppBar(
        backgroundColor: _color,
        title: Text(widget.title),
      ),
      body: const Body(),
    );
  }
}

class Body extends StatefulWidget {
  const Body({super.key});

  @override
  State<Body> createState() => _BodyState();
}

class _BodyState extends State<Body> {
  final ApiInterface repo = PotterRepository();
  late Future<List<CardData>?> data = repo.loadData(q: null);
  final TextEditingController searchController = TextEditingController();

  @override
  void initState() {
    super.initState();
    data = repo.loadData(q: null);
  }

  @override
  Widget build(BuildContext context) {

    return Padding(
      padding: EdgeInsets.only(top: MediaQuery.of(context).padding.top),
      child: Column(
        children: [
          Padding(
            padding: const EdgeInsets.all(12),
            child: CupertinoSearchTextField(
              controller: searchController,
              onChanged: (search) {
                setState(() {
                  data = repo.loadData(q: search.isNotEmpty ? search : null);
                });
              },
            ),
          ),
          Expanded(
              child: FutureBuilder<List<CardData>?>(
                future: data,
                builder: (context, snapshot) =>
                    SingleChildScrollView(
                        padding: const EdgeInsets.only(
                            left: 20, right: 20, top: 20),
                        child: snapshot.hasData
                            ? Column(
                          children: snapshot.data?.map((data) {
                            return _Card.fromData(
                              data,
                              onLike: (text, isLiked) =>
                                  _showSnackBar(context, text, isLiked),
                              onTap: () => _navToDetails(context, data),
                            );
                          }).toList() ??
                              [],
                        )
                            : const CircularProgressIndicator()
                    ),
              )
          )
        ],
      ),
    );
  }

  void _showSnackBar(BuildContext context, String title, bool isLiked) {
    WidgetsBinding.instance.addPostFrameCallback((_) {
      ScaffoldMessenger.of(context).showSnackBar(SnackBar(
        content: Text(
          'Article $title ${isLiked ? 'liked!' : 'disliked :('}',
          style: Theme
              .of(context)
              .textTheme
              .bodyLarge,),
        backgroundColor: Colors.orangeAccent,
        duration: const Duration(seconds: 1),
      ));
    });
  }

  void _navToDetails(BuildContext context, CardData data) {
    Navigator.push(
      context,
      CupertinoPageRoute(builder: (context) => DetailsPage(data)),
    );
  }


}